<!--- 
	csw-Image-Textblock-ImageAbove-genericRH.cfm
	
	This 'generic' render handler render handler implements the 'getGenericFieldMappingsArray' function below to describe
	the available field that can be mapped.
	
	The logic above the function, renders an image with a caption above it, and optionally renders a description 
	and link.
	
	Note that for 'generic' render handlers the mapped data, if any, will always be passed in the following structure:
		Attributes.ElementInfo.ElementData.PropertyValues
	This is an array of one or more items, with a sub-structure named 'values'.  
	Values is a structure with members as defined by the getGenericFieldMappingsArray() function.
	
	This generic render handler assumes these fields:
		- Title - Required
		- Description - Required
		- ImageURL (string or struct)
		- LinkURL  (string or struct)
		- LinkText  (string)
		
--->

<cfscript>
	// load Resources
	Application.ADF.scripts.loadResources("Bootstrap,cswImageTextBlockImageAboveCSS");

	// exit if all we're doing is detecting required resources
	if ( Request.RenderState.RenderMode EQ "getResources" )
		exit;

	if ( NOT structKeyExists(attributes,'elementinfo') )
		exit;

//WriteDump(var=attributes,expand=false);
//WriteDump(var=attributes.elementinfo.elementdata,expand=false);

	// Get Records from the ElementData structure
	elementData = Attributes.ElementInfo.ElementData;
	items = ArrayNew(1);

	// Get Records from the Element as an Array of Structures
	if ( StructKeyExists(elementData, 'PropertyValues') AND ArrayLen(elementData.PropertyValues) )
		items = elementData.PropertyValues;
	
	totalItems = ArrayLen(items);
	
	rhMetadata = StructNew();
	if ( !StructKeyExists(Attributes.ElementInfo,"RenderHandlerMetaData") )
	{
		WriteOutput("<div><em>Please bind the selected Layout Render Handler to the appropriate Widget Configuration Metadata Form!</em></div>");
	}	
	else
	{
		// Get Render Handler Metadata
		rhMetadata = Attributes.ElementInfo.RenderHandlerMetaData;
	}
	
	// DEV: is this error handling logic is needed? 
	/* if ( !IsStruct(rhMetadata) OR StructIsEmpty(rhMetadata) )
	{
		WriteOutput("<div><em>Please open the custom properties for this element and select options for this Widget!</em></div>");
	}*/

//WriteDump(var=items,expand=false,label="items");
//WriteDump(var=rhMetadata,expand=false,label="rhMetadata");
	
	// Set the Form and Field Names for Render Handler Metadata
	mdOptionsFieldName = "WidgetOptions";
	// Get the widget data from the bound Metadata Form with a "WidgetOptions" field
	widgetData = application.ADF.widgets.getWidgetConfigData(customMetdata=rhMetadata,configFieldName=mdOptionsFieldName);
	
//WriteDump(var=widgetData,expand=false,label="widgetData");	
	
	/* START - BUILD columnClass */
	columnClass = "";
	/* PARSE [Columns_XSmall] */
	if ( StructKeyExists(widgetData,"Columns_X_Small") )
		columnClass = application.ADF.widgets.buildClassValue(classStr=columnClass,newClass=widgetData.Columns_X_Small);

	/* PARSE [Columns_Small] */
	if ( StructKeyExists(widgetData,"Columns_Small") )
		columnClass = application.ADF.widgets.buildClassValue(classStr=columnClass,newClass=widgetData.Columns_Small);

	/* PARSE [Columns_Medium] */
	if ( StructKeyExists(widgetData,"Columns_Medium") )
		columnClass = application.ADF.widgets.buildClassValue(classStr=columnClass,newClass=widgetData.Columns_Medium);

	/* PARSE [Columns_Large] */
	if ( StructKeyExists(widgetData,"Columns_Large") )
		columnClass = application.ADF.widgets.buildClassValue(classStr=columnClass,newClass=widgetData.Columns_Large);
	/* END - BUILD columnClass */
	
	/* PARSE [Image_Align] */
	imgAlignClass = "";
	if ( StructKeyExists(widgetData,"Image_Align") )
		imgAlignClass = application.ADF.widgets.buildClassValue(classStr=imgAlignClass,newClass=widgetData.Image_Align);
	
	/* PARSE [Title_Tag] */
	titleTag = "H2";
	if ( StructKeyExists(widgetData,"Title_Tag") )
		titleTag = TRIM(widgetData.Title_Tag);	

	/* PARSE [Title_Align] */
	titleAlignClass = "";
	if ( StructKeyExists(widgetData,"Title_Align") )
		titleAlignClass = application.ADF.widgets.buildClassValue(classStr=titleAlignClass,newClass=widgetData.Title_Align);
	
	/* PARSE [Title_Color] */
	titleColorClass = "";
	if ( StructKeyExists(widgetData,"Title_Color") )
		titleColorClass = application.ADF.widgets.buildClassValue(classStr=titleColorClass,newClass=widgetData.Title_Color);
	
	/* PARSE [Description_Align] */
	descAlignClass = "";
	if ( StructKeyExists(widgetData,"Description_Align") )
		descAlignClass = application.ADF.widgets.buildClassValue(classStr=titleAlignClass,newClass=widgetData.Description_Align);
	
	/* PARSE [Description_Color] */
	descColorClass = "";
	if ( StructKeyExists(widgetData,"Description_Color") )
		descColorClass = application.ADF.widgets.buildClassValue(classStr=descColorClass,newClass=widgetData.Description_Color);
	
	/* PARSE [Button_Enabled] */
	btnEnabled = false;
	if ( StructKeyExists(widgetData,"Button_Enabled") AND IsBoolean(widgetData.Button_Enabled) AND widgetData.Button_Enabled )
		btnEnabled = true;
	
	/* PARSE [Button_Style] */
	btnStyleClass = "";
	btnSizeClass = "";
	if ( StructKeyExists(widgetData,"Button_Style") )
	{
		btnStyleClass = application.ADF.widgets.buildClassValue(classStr=btnStyleClass,newClass=widgetData.Button_Style);
		
		/* PARSE [Button_Size] - Button Style is defined */
		if ( StructKeyExists(widgetData,"Button_Size") AND widgetData.Button_Style NEQ "none" )
			btnSizeClass = application.ADF.widgets.buildClassValue(classStr=btnSizeClass,newClass=widgetData.Button_Size);
	}
	
	/* PARSE [Button_Align] */
	btnAlignClass = "";
	if ( StructKeyExists(widgetData,"Button_Align") )
		btnAlignClass = application.ADF.widgets.buildClassValue(classStr=btnAlignClass,newClass=widgetData.Button_Align);
	
	/* NOT CONFIGURED:
		Image Style;Title Style;Description Style;Button Color 
	*/

	// Build the Bootstrap ClearFix Struct from the columnClass list
	clearFixStruct = application.ADF.widgets.calcBootstrapClearFix(columnClass);
</cfscript>

<cfoutput>
	<cfif totalItems>
	<div class="csw-ImgTextblock-ImgAbove">
		
		<cfloop from="1" to="#totalItems#" index="itm">
		
		<!-- // Item: #itm# -->
		<cfif StructKeyExists(items[itm],'values')>
		
		<cfscript>
			// PARSE ImageURL Data
			imgURL = "";
			if ( StructKeyExists(items[itm].values,'ImageURL') AND isStruct(items[itm].values.ImageURL) )
			{
				// assume CommonSpot image field is used and structure is passed
				imgURL = items[itm].values.ImageURL.ResolvedURL.ServerRelative;
			}
			else
			{
				// assume img url string is passed 
				imgURL = items[itm].values.ImageURL;
			}
			
			// PARSE LinkURL Data
			linkURL = application.ADF.csData.parseCSURL(str=items[itm].values.LinkURL);
		</cfscript>
		
		<!--- // START: Image Textblock Image Above HTML --->
		<div class="#columnClass# cswItems cswItem-#itm#">
			<div class="cswTextblockBox">
			
				<div class="cswImgBox clearfix">
					<img class="img-responsive #imgAlignClass# cswImg" src="#imgURL#" alt="#items[itm].values.Title#">
				</div>
				
				<div class="#descAlignClass# cswTextBox">
				
					<#titleTag# class="#titleColorClass# #titleAlignClass# cswTitle">
						<cfif LEN(TRIM(linkURL))><a href="#linkURL#"></cfif>#items[itm].values.Title#<cfif LEN(TRIM(linkURL))></a></cfif>
					</#titleTag#>
					
					<p class="#descColorClass# cswDesc">
					#items[itm].values.description#
					</p>
					
					<cfif btnEnabled AND LEN(TRIM(linkURL))>
						<p class="#btnAlignClass# cswButtonBox"><a class="readmore #btnStyleClass# #btnSizeClass# cswButton" href="#linkURL#">#items[itm].values.linkText#</a></p>
					</cfif>
					
				</div>
				
			</div>
		</div>
		<!--- // END: Image Textblock Image Above HTML --->
		
		<!--- // Bootstrap ClearFix Generator ---> 
		<!--- // - Used to handle column rendering wrapping for diffent device sizes when heights/content of each item vary  --->
		#application.ADF.widgets.renderBootstrapClearFix(itm, clearFixStruct)#
		</cfif>
		
		</cfloop>
		
	</div>
	<div class="clearfix visible-xs"></div>	
	</cfif>
</cfoutput>

<!--- ////////////////////////////////////////////////////////////////////////// --->
<!--- //////////////////   GENERIC RENDER HANDLER FUNCTIONS   ////////////////// --->
<!--- ////////////////////////////////////////////////////////////////////////// --->

<!---
	This is the function that is called by CommonSpot when this render handler is registred, allowing teh user to 
	map the fields of the source element to this render handler.
	
	Here we expose build a structure describing the four (4) fields that can be mapped. Two (2) of the fields are marked 
	as required. The other two are optional.
		- Title - Required
		- Description - Required
		- Image  (string or csImage structure)
		- LinkURL  ((string or csExtURL structure)
		- LinkText (string)
	
	Each fieldMappingArray item is a structure with the following 3 fields:
		- FieldName
		- Description
		- Required
--->
<cffunction name="getGenericFieldMappingsArray" returntype="array" access="private" output="no">
	<cfscript>
		var fieldMappingArray = ArrayNew(1);
		var fieldDetailsStruct = StructNew();
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Title';
		fieldDetailsStruct.Description = 'The Title for the Image Text Block.';
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Description';
		fieldDetailsStruct.Description = 'The Description for the Icon Text Block.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'ImageURL';
		fieldDetailsStruct.Description = 'The server relative URL to the image, or the CommonSpot Image field structure.';
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'LinkURL';
		fieldDetailsStruct.Description = 'The URL visitors will be navigated to when clicking the Link in the Image Text Block. If not provided the Image Text Block will not be linked.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'LinkText';
		fieldDetailsStruct.Description = 'The text that will render for the Link or Button in the Image Text Block. If not provided the Image Text Block will not be linked.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		return fieldMappingArray;
	</cfscript>
</cffunction>
